package com.bureau.base

import android.content.Context
import android.content.pm.ApplicationInfo
import android.content.pm.PackageInfo
import android.content.pm.PackageManager

class AppInfo(private val context: Context) {
    private fun getPackageInfo(): PackageInfo? {
        return try {
            context.packageManager.getPackageInfo(context.packageName, 0)
        } catch (e: PackageManager.NameNotFoundException) {
            null
        }
    }

    fun getPackageName(): String {
        return getPackageInfo()?.packageName ?: ""
    }

    fun getVersion(): String {
        val packageInfo = getPackageInfo()
        return (packageInfo?.versionName ?: "") + "+" + (packageInfo?.versionCode ?: "")
    }

    fun getAppName(): String {
        val applicationInfo: ApplicationInfo = context.applicationInfo
        return applicationInfo.loadLabel(context.packageManager).toString()
    }

    fun isInDebugMode(): Boolean {
        return try {
            val appInfo: ApplicationInfo =
                context.packageManager.getApplicationInfo(context.packageName, 0)
            0 != appInfo.flags and ApplicationInfo.FLAG_DEBUGGABLE
        } catch (e: PackageManager.NameNotFoundException) {
            false
        }
    }
}
